/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0 and the Server Side Public License, v 1; you may not use this file except
 * in compliance with, at your election, the Elastic License 2.0 or the Server
 * Side Public License, v 1.
 */

import { ReactElement } from 'react';
import { FilterManager } from '@kbn/data-plugin/public';
import { createFilterManagerMock } from '@kbn/data-plugin/public/query/filter_manager/filter_manager.mock';
import { SearchInput } from '..';
import { getSavedSearchUrl } from '@kbn/saved-search-plugin/public';
import { DiscoverServices } from '../build_services';
import { discoverServiceMock } from '../__mocks__/services';
import { SavedSearchEmbeddable, SearchEmbeddableConfig } from './saved_search_embeddable';
import { render } from 'react-dom';
import { createSearchSourceMock } from '@kbn/data-plugin/public/mocks';
import { Observable, of, throwError } from 'rxjs';
import { ReactWrapper } from 'enzyme';
import { SHOW_FIELD_STATISTICS } from '../../common';
import { IUiSettingsClient } from '@kbn/core-ui-settings-browser';
import { SavedSearchEmbeddableComponent } from './saved_search_embeddable_component';
import { VIEW_MODE } from '../../common/constants';
import { buildDataViewMock, deepMockedFields } from '../__mocks__/data_view';

let discoverComponent: ReactWrapper;

jest.mock('react-dom', () => {
  const { mount } = jest.requireActual('enzyme');
  return {
    ...jest.requireActual('react-dom'),
    render: jest.fn((component: ReactElement) => {
      discoverComponent = mount(component);
    }),
  };
});

const waitOneTick = () => new Promise((resolve) => setTimeout(resolve, 0));
function getSearchResponse(nrOfHits: number) {
  const hits = new Array(nrOfHits).map((idx) => ({ id: idx }));
  return of({
    rawResponse: {
      hits: { hits, total: nrOfHits },
    },
    isPartial: false,
    isRunning: false,
  });
}

const dataViewMock = buildDataViewMock({ name: 'the-data-view', fields: deepMockedFields });

describe('saved search embeddable', () => {
  let mountpoint: HTMLDivElement;
  let filterManagerMock: jest.Mocked<FilterManager>;
  let servicesMock: jest.Mocked<DiscoverServices>;

  let executeTriggerActions: jest.Mock;
  let showFieldStatisticsMockValue: boolean = false;
  let viewModeMockValue: VIEW_MODE = VIEW_MODE.DOCUMENT_LEVEL;

  const createEmbeddable = (searchMock?: jest.Mock, customTitle?: string) => {
    const searchSource = createSearchSourceMock({ index: dataViewMock }, undefined, searchMock);
    const savedSearchMock = {
      id: 'mock-id',
      title: 'saved search',
      sort: [['message', 'asc']] as Array<[string, string]>,
      searchSource,
      viewMode: viewModeMockValue,
    };

    const url = getSavedSearchUrl(savedSearchMock.id);
    const editUrl = `/app/discover${url}`;
    const indexPatterns = [dataViewMock];
    const savedSearchEmbeddableConfig: SearchEmbeddableConfig = {
      savedSearch: savedSearchMock,
      editUrl,
      editPath: url,
      editable: true,
      indexPatterns,
      filterManager: filterManagerMock,
      services: servicesMock,
    };
    const searchInput: SearchInput = {
      id: 'mock-embeddable-id',
      timeRange: { from: 'now-15m', to: 'now' },
      columns: ['message', 'extension'],
      rowHeight: 30,
      rowsPerPage: 50,
    };
    if (customTitle) {
      searchInput.title = customTitle;
    }

    executeTriggerActions = jest.fn();

    const embeddable = new SavedSearchEmbeddable(
      savedSearchEmbeddableConfig,
      searchInput,
      executeTriggerActions
    );

    // this helps to trigger reload
    // eslint-disable-next-line dot-notation
    embeddable['inputSubject'].next = jest.fn(
      (input) => (input.lastReloadRequestTime = Date.now())
    );

    return { embeddable, searchInput, searchSource };
  };

  beforeEach(() => {
    mountpoint = document.createElement('div');
    filterManagerMock = createFilterManagerMock();

    showFieldStatisticsMockValue = false;
    viewModeMockValue = VIEW_MODE.DOCUMENT_LEVEL;

    servicesMock = discoverServiceMock as unknown as jest.Mocked<DiscoverServices>;

    (servicesMock.uiSettings as unknown as jest.Mocked<IUiSettingsClient>).get.mockImplementation(
      (key: string) => {
        if (key === SHOW_FIELD_STATISTICS) return showFieldStatisticsMockValue;
      }
    );
  });

  afterEach(() => {
    mountpoint.remove();
    jest.resetAllMocks();
  });

  it('should update input correctly', async () => {
    const { embeddable } = createEmbeddable();
    jest.spyOn(embeddable, 'updateOutput');

    embeddable.render(mountpoint);
    expect(render).toHaveBeenCalledTimes(1);
    await waitOneTick();
    expect(render).toHaveBeenCalledTimes(2);

    const searchProps = discoverComponent.find(SavedSearchEmbeddableComponent).prop('searchProps');

    searchProps.onAddColumn!('bytes');
    await waitOneTick();
    expect(searchProps.columns).toEqual(['message', 'extension', 'bytes']);
    expect(render).toHaveBeenCalledTimes(4); // twice per an update to show and then hide a loading indicator

    searchProps.onRemoveColumn!('bytes');
    await waitOneTick();
    expect(searchProps.columns).toEqual(['message', 'extension']);

    searchProps.onSetColumns!(['message', 'bytes', 'extension'], false);
    await waitOneTick();
    expect(searchProps.columns).toEqual(['message', 'bytes', 'extension']);

    searchProps.onMoveColumn!('bytes', 2);
    await waitOneTick();
    expect(searchProps.columns).toEqual(['message', 'extension', 'bytes']);

    expect(searchProps.rowHeightState).toEqual(30);
    searchProps.onUpdateRowHeight!(40);
    await waitOneTick();
    expect(searchProps.rowHeightState).toEqual(40);

    expect(searchProps.rowsPerPageState).toEqual(50);
    searchProps.onUpdateRowsPerPage!(100);
    await waitOneTick();
    expect(searchProps.rowsPerPageState).toEqual(100);

    searchProps.onFilter!({ name: 'customer_id', type: 'string', scripted: false }, [17], '+');
    await waitOneTick();
    expect(executeTriggerActions).toHaveBeenCalled();
  });

  it('should render saved search embeddable when successfully loading data', async () => {
    // mock return data
    const search = jest.fn().mockReturnValue(getSearchResponse(1));
    const { embeddable } = createEmbeddable(search);
    jest.spyOn(embeddable, 'updateOutput');

    // check that loading state
    const loadingOutput = embeddable.getOutput();
    expect(loadingOutput.loading).toBe(true);
    expect(loadingOutput.rendered).toBe(false);
    expect(loadingOutput.error).toBe(undefined);

    embeddable.render(mountpoint);
    expect(render).toHaveBeenCalledTimes(1);

    // wait for data fetching
    await waitOneTick();
    expect(render).toHaveBeenCalledTimes(2);

    // check that loading state
    const loadedOutput = embeddable.getOutput();
    expect(loadedOutput.loading).toBe(false);
    expect(loadedOutput.rendered).toBe(true);
    expect(loadedOutput.error).toBe(undefined);
  });

  it('should render saved search embeddable when empty data is returned', async () => {
    // mock return data
    const search = jest.fn().mockReturnValue(getSearchResponse(0));
    const { embeddable } = createEmbeddable(search);
    jest.spyOn(embeddable, 'updateOutput');

    // check that loading state
    const loadingOutput = embeddable.getOutput();
    expect(loadingOutput.loading).toBe(true);
    expect(loadingOutput.rendered).toBe(false);
    expect(loadingOutput.error).toBe(undefined);

    embeddable.render(mountpoint);
    expect(render).toHaveBeenCalledTimes(1);

    // wait for data fetching
    await waitOneTick();
    expect(render).toHaveBeenCalledTimes(2);

    // check that loading state
    const loadedOutput = embeddable.getOutput();
    expect(loadedOutput.loading).toBe(false);
    expect(loadedOutput.rendered).toBe(true);
    expect(loadedOutput.error).toBe(undefined);
  });

  it('should render in AGGREGATED_LEVEL view mode', async () => {
    showFieldStatisticsMockValue = true;
    viewModeMockValue = VIEW_MODE.AGGREGATED_LEVEL;

    const { embeddable } = createEmbeddable();
    jest.spyOn(embeddable, 'updateOutput');

    // check that loading state
    const loadingOutput = embeddable.getOutput();
    expect(loadingOutput.loading).toBe(true);
    expect(loadingOutput.rendered).toBe(false);
    expect(loadingOutput.error).toBe(undefined);

    embeddable.render(mountpoint);
    expect(render).toHaveBeenCalledTimes(1);

    // wait for data fetching
    await waitOneTick();
    expect(render).toHaveBeenCalledTimes(2);

    // check that loading state
    const loadedOutput = embeddable.getOutput();
    expect(loadedOutput.loading).toBe(false);
    expect(loadedOutput.rendered).toBe(true);
    expect(loadedOutput.error).toBe(undefined);
  });

  it('should emit error output in case of fetch error', async () => {
    const search = jest.fn().mockReturnValue(throwError(new Error('Fetch error')));
    const { embeddable } = createEmbeddable(search);
    jest.spyOn(embeddable, 'updateOutput');

    embeddable.render(mountpoint);
    // wait for data fetching
    await waitOneTick();

    expect((embeddable.updateOutput as jest.Mock).mock.calls[1][0].error.message).toBe(
      'Fetch error'
    );
    // check that loading state
    const loadedOutput = embeddable.getOutput();
    expect(loadedOutput.loading).toBe(false);
    expect(loadedOutput.rendered).toBe(true);
    expect(loadedOutput.error).not.toBe(undefined);
  });

  it('should not fetch data if only a new input title is set', async () => {
    const search = jest.fn().mockReturnValue(getSearchResponse(1));
    const { embeddable, searchInput } = createEmbeddable(search);

    embeddable.render(mountpoint);
    // wait for data fetching
    await waitOneTick();
    expect(search).toHaveBeenCalledTimes(1);
    embeddable.updateOutput({ title: 'custom title' });
    embeddable.updateInput(searchInput);
    await waitOneTick();
    expect(search).toHaveBeenCalledTimes(1);
  });
  it('should not reload when the input title doesnt change', async () => {
    const search = jest.fn().mockReturnValue(getSearchResponse(1));
    const { embeddable } = createEmbeddable(search, 'custom title');
    embeddable.reload = jest.fn();
    embeddable.render(mountpoint);
    // wait for data fetching
    await waitOneTick();
    embeddable.updateOutput({ title: 'custom title' });
    await waitOneTick();

    expect(embeddable.reload).toHaveBeenCalledTimes(0);
    expect(search).toHaveBeenCalledTimes(1);
  });

  it('should reload when a different input title is set', async () => {
    const search = jest.fn().mockReturnValue(getSearchResponse(1));
    const { embeddable } = createEmbeddable(search, 'custom title');
    embeddable.reload = jest.fn();
    embeddable.render(mountpoint);

    await waitOneTick();
    embeddable.updateOutput({ title: 'custom title changed' });
    await waitOneTick();

    expect(embeddable.reload).toHaveBeenCalledTimes(1);
    expect(search).toHaveBeenCalledTimes(1);
  });

  it('should not reload and fetch when a input title matches the saved search title', async () => {
    const search = jest.fn().mockReturnValue(getSearchResponse(1));
    const { embeddable } = createEmbeddable(search);
    embeddable.reload = jest.fn();
    embeddable.render(mountpoint);
    await waitOneTick();
    embeddable.updateOutput({ title: 'saved search' });
    await waitOneTick();

    expect(embeddable.reload).toHaveBeenCalledTimes(0);
    expect(search).toHaveBeenCalledTimes(1);
  });

  it('should correctly handle aborted requests', async () => {
    const { embeddable, searchSource } = createEmbeddable();
    await waitOneTick();
    const updateOutput = jest.spyOn(embeddable, 'updateOutput');
    const abortSignals: AbortSignal[] = [];
    jest.spyOn(searchSource, 'fetch$').mockImplementation(
      (options) =>
        new Observable(() => {
          if (options?.abortSignal) {
            abortSignals.push(options.abortSignal);
          }
          throw new Error('Search failed');
        })
    );
    embeddable.reload();
    embeddable.reload();
    await waitOneTick();
    expect(updateOutput).toHaveBeenCalledTimes(3);
    expect(abortSignals[0].aborted).toBe(true);
    expect(abortSignals[1].aborted).toBe(false);
    embeddable.reload();
    await waitOneTick();
    expect(updateOutput).toHaveBeenCalledTimes(5);
    expect(abortSignals[2].aborted).toBe(false);
  });
});
